/*********************************************************************
**   Martial Chateauvieux
**   HORUS UAV System - Mega128 Dev board
**   19 November 2008
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

/** \file config-mega128-dev-board-100.h
 **

 ** \par Introduction
 **
 ** The config.h file groups all symbolic definitions used by
 ** peripheral drivers. Used together with a set of macro files
 ** (macro_*.h), they make it easy to create or port peripheral
 ** drivers for different AVR mcu projects.

 ** The file is located in a directory visible from all projects. It's
 ** (real) name gives indication of the hardware environnment it
 ** describes. It is linked from the project using a symbolic link
 ** called "config.h".

 ** \par config-mega128-dev-board-100.h

 **
 ********************************************************************/

#ifndef __CONFIG_MEGA128_DEV_H__
#define __CONFIG_MEGA128_DEV_H__

#include <avr/io.h>

#include "macros_uart.h"

/** \def SYSCLK
 **
 ** \brief CPU clock frequency in Hz.

 ** \obsolete

 ** This is used in most "dispatch loop" file implementations. It
 ** tends to be replaced by the following F_CPU, as this is the macro
 ** required for the standard C library for the dela functions.
 **
 **/
//#define SYSCLK	3686400ul	// clock ticks/s
#define SYSCLK	7372800ul	// clock ticks/s

/** \def F_CPU
 ** \brief CPU clock frequency in Hz.
 **/
#define F_CPU	SYSCLK

/*==========================================================
** UART 0 config (GPS) 
** - The UART0 on the mega128 uses the pins PE0 and PE1.
**========================================================*/
//#define USE_UART0
#define UART0_BAUDRT		HUS_UART_BRATE_57600
#define UART0_PARITY		HUS_UART_PAR_NONE
#define UART0_STPBIT		HUS_UART_STPBIT_1
#define UART0_INPBUFSIZE	8
#define UART0_OUTPBUFSIZE	128
/*========================================================*/

/*==========================================================
** UART 1 config (RF link / PC config)
** - The UART1 on the mega128 uses the pins PD2 and PD3
**========================================================*/
//#define USE_UART1
#define UART1_BAUDRT		HUS_UART_BRATE_9600
#define UART1_INPBUFSIZE	8
#define UART1_OUTPBUFSIZE	64
/*========================================================*/

/*==========================================================
** ICP config
** - The ICP of the timer3.
**========================================================*/
//#define USE_ICP3
#ifdef USE_ICP3
#define USE_TIMER3
#define ICP3_REGISTER	ICR3

#define ICP3_PIN_PORT	PORTE
#define ICP3_PIN_DDR	DDRE
#define ICP3_PIN_BIT	7
#endif /* USE_ICP3 */
/*========================================================*/

/*==========================================================
** OCM config
** - The OCM A of the timer3.
**========================================================*/
//#define USE_OC3A
#ifdef USE_OC3A

#define USE_TIMER3
#define OC3A_REGISTER	OCR3A

#define OC3A_PIN_PORT	PORTE
#define OC3A_PIN_DDR	DDRE
#define OC3A_PIN_BIT	3

#define OC3B_REGISTER	OCR3B

#define OC3B_PIN_PORT	PORTE
#define OC3B_PIN_DDR	DDRE
#define OC3B_PIN_BIT	4

#define OC3C_REGISTER	OCR3C

#define OC3C_PIN_PORT	PORTE
#define OC3C_PIN_DDR	DDRE
#define OC3C_PIN_BIT	5

#endif /* USE_OC3 */
/*========================================================*/

/*==========================================================
** ADC config
** - The ADC of the mega128 uses the pins PF0-7
**========================================================*/
#define USE_ADC
#ifdef USE_ADC
#define AIRSPEED_ADCPIN	ADC0
#endif
/*========================================================*/


/*==========================================================
** Leds & Buttons 	(Header PORTC)
**========================================================*/
#define DLED_0_PORT		PORTC
#define DLED_0_DDR		DDRC
#define DLED_0_FIRST_BIT	0

#define DLED_1_PORT		PORTC
#define DLED_1_DDR		DDRC
#define DLED_1_FIRST_BIT	2

#define MAX_LED_IDX		2
#define MAX_BUTTON_IDX		0
/*========================================================*/



/*==========================================================
** LCD 			(Header PORTB)
**========================================================*/

#define USE_DIP204
#if defined(USE_DIP204) 
#define LCD_CONTROLLER_KS0073	// Controller for DIP204 LCDs

#define LCD_ROWS	4
#define LCD_COLS	20

#define LCD_ROW_0_DDRAMADR	0x00
#define LCD_ROW_1_DDRAMADR	0x20
#define LCD_ROW_2_DDRAMADR	0x40
#define LCD_ROW_3_DDRAMADR	0x60
#endif



//#define USE_ANALOGDISPLAY4x20

#define HUS_LCD_USEBUSYPIN		// define to use the busy
					// pin. Comment out to use a
					// fix loop timer instead.

#define __HUS_LCD_PORT	PORTB
#define __HUS_LCD_DDR	DDRB
#define __HUS_LCD_PIN	PINB

#define	HUS_LCD_RS_BIT		0
#define HUS_LCD_RS_PORT		__HUS_LCD_PORT	//
#define	HUS_LCD_RS_DDR		__HUS_LCD_DDR	//

#define	HUS_LCD_RW_BIT		1
#define HUS_LCD_RW_PORT		__HUS_LCD_PORT	//
#define	HUS_LCD_RW_DDR		__HUS_LCD_DDR	//

#define	HUS_LCD_E_BIT		2
#define HUS_LCD_E_PORT		__HUS_LCD_PORT	//
#define	HUS_LCD_E_DDR		__HUS_LCD_DDR	//

#define HUS_LCD_DB4_BIT		3		//
#define HUS_LCD_DB4_PORT	__HUS_LCD_PORT	//
#define HUS_LCD_DB4_PIN		__HUS_LCD_PIN	//
#define	HUS_LCD_DB4_DDR		__HUS_LCD_DDR	//

#define HUS_LCD_DB5_BIT		4		//
#define HUS_LCD_DB5_PORT	__HUS_LCD_PORT	//
#define HUS_LCD_DB5_PIN		__HUS_LCD_PIN	//
#define	HUS_LCD_DB5_DDR		__HUS_LCD_DDR	//

#define HUS_LCD_DB6_BIT		5		//
#define HUS_LCD_DB6_PORT	__HUS_LCD_PORT	//
#define HUS_LCD_DB6_PIN		__HUS_LCD_PIN	//
#define	HUS_LCD_DB6_DDR		__HUS_LCD_DDR	//

#define HUS_LCD_DB7_BIT		6		//
#define HUS_LCD_DB7_PORT	__HUS_LCD_PORT	//
#define HUS_LCD_DB7_PIN		__HUS_LCD_PIN	//
#define	HUS_LCD_DB7_DDR		__HUS_LCD_DDR	//
/*========================================================*/

#endif  /*__CONFIG_MEGA128_DEV_H__ */
