/*********************************************************************
**   Martial Chateauvieux
**   HORUS UAV System
**   07 March 2007
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

/** \file

 ** The drv_adc.c module implements the adc driver.

*********************************************************************/

#include <stdio.h>
#include <avr/interrupt.h>

#include "drv_adc.h"

//volatile HUS_adc_callback_t adc_callback = NULL;

void HUS_adc_config_common(//const HUS_adc_callback_t callback,
			   const uint8_t channel_mask) {

  HUS_adc_vref(HUS_ADC_REF_EXTERN);
  HUS_adc_prescaler(HUS_ADC_PRESC_128);

  HUS_adc_align_right();

  // Selects ADC channel
  ADMUX &= ~(_BV(MUX4)|_BV(MUX3)|_BV(MUX2)|_BV(MUX1)|_BV(MUX0));
  ADMUX |= channel_mask;
  
  //adc_callback = callback;
  HUS_adc_enable_intr();
}


void HUS_adc_init_ADC0() {//HUS_adc_callback_t callback) {
  HUS_adc_config_common(//callback,
			0);
}

void HUS_adc_init_diff_ADC0_ADC0_x10() {//HUS_adc_callback_t callback) {
  HUS_adc_config_common(//callback,
			_BV(MUX3));
}

void HUS_adc_init_diff_ADC0_ADC1_x10() {//HUS_adc_callback_t callback) {
  HUS_adc_config_common(//callback, 
			(_BV(MUX3)|_BV(MUX0)));
}

void HUS_adc_init_diff_ADC0_ADC0_x200() {//HUS_adc_callback_t callback) {
  HUS_adc_config_common(//callback,
			(_BV(MUX3)|_BV(MUX1)));
}

void HUS_adc_init_diff_ADC0_ADC1_x200() {//HUS_adc_callback_t callback) {
  HUS_adc_config_common(//callback,
			(_BV(MUX3)|_BV(MUX1)|_BV(MUX0)));
}

int32_t HUS_adc_se_to_voltx10e6(int32_t adc_value) {
  // Single ended conversion: (ADCValue * VRef * 10e6) / 1024
  return ((adc_value * 5 * 1000000) / 1024);
}
int32_t HUS_adc_dif_to_voltx10e6(int32_t adc_value, uint8_t adc_gain) {
  return adc_value * 5 * (1000000 / (adc_gain * 512));
}
