/*********************************************************************
** Martial Chateauvieux
** Horus - Wind speed sensor driver
** 03 January 2008
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2007 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

#include <string.h>

#include <avr/io.h>
#include <avr/interrupt.h>

#include "config.h"
#include "project_config.h"

#include "drv_airspeed.h"
#include "mydelay.h"


/*==========================================================
** ADC Conversion Complete interrupt handler
**---------------------------------------------------------
**========================================================*/
ISR(ADC_vect) {
  HUS_AIRS_ADC_ISR(AIRS_GLOBAL);
}
/*========================================================*/

void HUS_AIRS_init(volatile HUS_AIRS_t *airs_data) {
  memset((HUS_AIRS_t *)airs_data, sizeof(*airs_data), 0);
  HUS_adc_init_ADC0();
  HUS_adc_enable_intr();
}

void    HUS_AIRS_set_adc_offset(volatile HUS_AIRS_t *airs_data, const int16_t offset) {
  airs_data->offset = offset;
}
int16_t HUS_AIRS_get_adc_offset(volatile const HUS_AIRS_t *airs_data) {
  return airs_data->offset;
}

void HUS_AIRS_restart_conversion(volatile HUS_AIRS_t *airs_data) {
	// Restart conversion
  airs_data->tmp_value = 
    airs_data->mean_value = 
    airs_data->adc_status.ready =
    airs_data->count = 0;
  
  HUS_adc_on();
  HUS_adc_start();
}

int32_t HUS_AIRS_adc_to_airspeed_ms(int32_t adc_value) {

      int32_t airspeed_ms;
      if (adc_value < 352) {
	airspeed_ms = ((adc_value) / 3) - 112;
      } else if (adc_value < 512) {
	airspeed_ms = ((adc_value) / 14) - 19;
      } else {
	airspeed_ms = ((adc_value) / 31) - 2;
      }
      return airspeed_ms;
}
int32_t HUS_AIRS_adc_to_airspeed_dms(int32_t adc_value) {

      int32_t airspeed_dms;
      if (adc_value < 352) {
	airspeed_dms = ((adc_value*10) / 3) - 112*10;
      } else if (adc_value < 512) {
	airspeed_dms = ((adc_value*10) / 14) - 19*10;
      } else {
	airspeed_dms = ((adc_value*10) / 31) - 2*10;
      }
      return airspeed_dms;
}

void HUS_AIRS_calibrate_adc(volatile HUS_AIRS_t *airspeed_status) {

#define HUS_AIRS_ADCINITLOOPS	(1<<10)
  uint32_t adc_offset = 0;
  uint32_t i;

  for (i = 0 ; i < HUS_AIRS_ADCINITLOOPS ; i++) {
    HUS_AIRS_restart_conversion(airspeed_status);
    while(airspeed_status->adc_status.ready != 1) {  
      delay_us(2);
    }
    adc_offset += airspeed_status->mean_value;
  }
  adc_offset /= HUS_AIRS_ADCINITLOOPS;
  airspeed_status->offset = 337 - adc_offset;
}


