/*********************************************************************
 ** Martial Chateauvieux
 ** Horus - Wind speed sensor driver
 ** 03 January 2008
 **********************************************************************
 **
 **    HORUS UAV System V1.1.0 - Copyright (C) 2003-2007 Martial
 **    Chateauvieux
 **
 **    This file is part of the HORUS UAV System distribution.
 **
 **    HORUS UAV System is free software; you can redistribute it
 **    and/or modify it under the terms of the GNU General Public
 **    License as published by the Free Software Foundation; either
 **    version 2 of the License, or (at your option) any later version.
 **
 **    HORUS UAV System is distributed in the hope that it will be
 **    useful, but WITHOUT ANY WARRANTY; without even the implied
 **    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 **    See the GNU General Public License for more details.
 **
 **    You should have received a copy of the GNU General Public
 **    License along with the HORUS UAV System; if not, write to the
 **    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 **    Boston, MA 02111-1307 USA
 **
 *********************************************************************/

#ifndef __DRV_AIRSPEED_H__
#define __DRV_AIRSPEED_H__

#include <inttypes.h>

#include "config.h"
#include "drv_adc.h"

typedef struct {
  HUS_adc_status_t adc_status;
  int16_t          tmp_value;
  uint8_t          count;
  int16_t          offset;
  int32_t	   mean_value;
} HUS_AIRS_t;

//extern volatile HUS_AIRS_t HUS_AIRS_status;

/**
 ** \brief Setup all values of the airspeed structure to 0
 */
void HUS_AIRS_init(volatile HUS_AIRS_t *airs_data);

/**
 ** \brief Set the offset
 */
void HUS_AIRS_set_adc_offset(volatile HUS_AIRS_t *airs_data, const int16_t adc_offset);


/**
 ** \brief Restart the conversion
 */
void HUS_AIRS_restart_conversion(volatile HUS_AIRS_t *airs_data);


/**
 ** \brief Convert adc_value to m/s air speed
 */
int32_t HUS_AIRS_adc_to_airspeed_ms(int32_t adc_value);

/**
 ** \brief Convert adc_value to dm/s air speed
 */
int32_t HUS_AIRS_adc_to_airspeed_dms(int32_t adc_value);

/**
 ** \brief Calibrates the ADC
 */
void HUS_AIRS_calibrate_adc(volatile HUS_AIRS_t *airspeed_status);

/**
 ** \brief ADC_ISR routine, with averaging
 */
#define HUS_AIRS_CONVERSIONBATCH 4
#define HUS_AIRS_ADC_ISR(status)  static volatile uint8_t idx = 0;			\
  switch(idx++) {					\
  case 0:						\
    HUS_adc_start();					\
    break;						\
  case 1:						\
    status.tmp_value   = (HUS_adc_read()).uvalue10b;	\
    HUS_adc_start();					\
    break;						\
  case HUS_AIRS_CONVERSIONBATCH:			\
    status.tmp_value  = (HUS_adc_read()).uvalue10b;	\
    status.mean_value = (status.tmp_value / HUS_AIRS_CONVERSIONBATCH) + status.offset;	\
    status.adc_status.ready = 1;			\
    idx = 0;						\
    HUS_adc_off();					\
    break;						\
  default:						\
    status.tmp_value   = (HUS_adc_read()).uvalue10b;	\
    HUS_adc_start();					\
    break;						\
  };


#endif /* __DRV_AIRSPEED_H__ */
