/*********************************************************************
**	Martial Chateauvieux
**	HD 44870 Compatible LCD Driver
**	23 Mai 2007
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

/** \file
 **   
 ** This file implements an universal HD44780 based Liquid Crystal
 ** Display.
 **
 ********************************************************************/

#ifndef __DRV_LCD_H__
#define __DRV_LCD_H__

#include <inttypes.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "config.h"

/*.... ANALOG DISPLAY 4x20 LCD DEFINITIONS .....*/
#ifdef USE_ANALOGDISPLAY4x20

#define HUS_LCD_CONTROLLER_HD44780

#define HUS_LCD_ROWS	4
#define HUS_LCD_COLS	20

#endif


/*.... DIP204 LCD DEFINITIONS .....*/
#ifdef USE_DIP204

#define HUS_LCD_CONTROLLER_KS0073	// Controller for DIP204 LCDs

#define HUS_LCD_ROWS	4
#define HUS_LCD_COLS	20

#define HUS_LCD_ROW_0_DDRAMADR	0x00
#define HUS_LCD_ROW_1_DDRAMADR	0x20
#define HUS_LCD_ROW_2_DDRAMADR	0x40
#define HUS_LCD_ROW_3_DDRAMADR	0x60

#endif /* USE_DIP204 */


#define HUS_LCD_LINE_RAMLEN_2ROWS (0x40) // The RAM length of a line
#define HUS_LCD_LINE_RAMLEN_4ROWS (0x20) // The RAM length of a line

#define HUS_LCD_BUSY_BIT	7	// This used only virtually
					// son the byte returned by
					// lcd_read()

/*--------------------------------------------------
** LCD Commands
**------------------------------------------------*/
/*--- Clear display -----------------------*/
#define HUS_LCD_CLR		((char)0x01)

/*--- Go home -----------------------------*/
#define HUS_LCD_HOME		((char)0x02)

/*--- Entry mode --------------------------*/
#define HUS_LCD_MODE_CMD	((char)0x04)
#define HUS_LCD_MODE_INC	((char)(HUS_LCD_MODE_CMD|0x02))
#define HUS_LCD_MODE_SFT	((char)(HUS_LCD_MODE_CMD|0x01))

/*--- On/Off for Dislpay/Cursor/Blinker ---*/
#define HUS_LCD_ON_CMD		((char)0x08)
#define HUS_LCD_ON_OFF		((char)(HUS_LCD_ON_CMD|0x00))
#define HUS_LCD_ON_DSPL		((char)(HUS_LCD_ON_CMD|0x04))
#define HUS_LCD_ON_CURS		((char)(HUS_LCD_ON_CMD|0x02))
#define HUS_LCD_ON_BLIK		((char)(HUS_LCD_ON_CMD|0x01))

/*--- Shift Cursor or (Display & Cursor) --*/
/*--- Commands cannot be combined ---------*/
#define HUS_LCD_SHFT_CMD  	((char)0x10)
#define HUS_LCD_SHFT_CLFT 	((char)(HUS_LCD_SHFT_CMD|0x00))
#define HUS_LCD_SHFT_CRGT 	((char)(HUS_LCD_SHFT_CMD|0x04))
#define HUS_LCD_SHFT_DLFT 	((char)(HUS_LCD_SHFT_CMD|0x08))
#define HUS_LCD_SHFT_DRGT 	((char)(HUS_LCD_SHFT_CMD|0x0C))

/*--- Set DDRAM ---------------------------*/
#define HUS_LCD_DDRAM_CMD	((char)0x80)

/* #define HUS_LCD_DDRAM_LINE0 	(0*HUS_LCD_LINE_RAMLEN) */
/* #define HUS_LCD_DDRAM_LINE1 	(1*HUS_LCD_LINE_RAMLEN) */
/* #define HUS_LCD_DDRAM_LINE2 	(2*HUS_LCD_LINE_RAMLEN) */
/* #define HUS_LCD_DDRAM_LINE3 	(3*HUS_LCD_LINE_RAMLEN) */


/*--- Set CGRAM ---------------------------*/
#define HUS_LCD_CGRAM	((char)0x40)

/*--------------------------------------------------
** lcd type struct
**------------------------------------------------*/
typedef struct {
    uint8_t rows:3; // 1-4
    uint8_t cols:5; // 8-20
} HUS_lcd_type_t;

/*--------------------------------------------------
** lcd status struct
**------------------------------------------------*/
typedef struct {
    uint8_t pos;  // 0-79
    HUS_lcd_type_t type;
} HUS_lcd_status_t;

/*--------------------------------------------------
** Functions
**------------------------------------------------*/
void HUS_lcd_init(const HUS_lcd_type_t *lcd_type);

#define HUS_LCD_CMD		1
#define HUS_LCD_DATA		0
extern void HUS_lcd_write(const char data, const uint8_t cmdflg);

#define HUS_LCD_BUSY_FLAG	1
extern char HUS_lcd_read(const uint8_t busy);

static inline void HUS_lcd_data(const char data) {HUS_lcd_write(data, HUS_LCD_DATA);}
static inline void HUS_lcd_cmd(const char cmd)   {HUS_lcd_write(cmd, HUS_LCD_CMD);}

extern void HUS_lcd_goto(const uint8_t row, const uint8_t col);
extern int  HUS_lcd_putc(const char c, FILE *stream);
extern void HUS_lcd_puts(const char *s, FILE *stream);
extern void HUS_lcd_puts2(const char *s, FILE *stream);
extern void HUS_lcd_putsn(const char *s, int len, FILE *stream);

void HUS_lcd_puts_P(PGM_P s, FILE *stream);
void HUS_lcd_on();
void HUS_lcd_off();
void HUS_lcd_cls();

#endif /* __DRV_LCD_H__ */
