/*********************************************************************
**   Martial Chateauvieux
**   HORUS UAV System
**   13 Feb 2007
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

/** \file
 **   
 ** This file implements a generic interface to manipulate the leds
 ** configured in the config.h file.
 **
 ** The implementation relies heavily on a project specific config.h
 ** file that must be included at the same time, to be useful.
 **
 ** This file should be symbolicly link from the project directory
 ** instead of being copied.
 **
 ********************************************************************/

#ifndef __LED_DRV_H__
#define __LED_DRV_H__

#include "project_config.h"
#include "hw_config.h"

#include "macros_led.h"


static inline void HUS_led_init() {
#ifdef LED_0_PORT // Simple LED
  HUS_LEDINIT(0);
#endif
#ifdef DLED_0_PORT // Duo LED
  HUS_DLEDINIT(0);
#endif

#ifdef LED_1_PORT // Simple LED
  HUS_LEDINIT(1);
#endif
#ifdef DLED_1_PORT // Duo LED
  HUS_DLEDINIT(1);
#endif

#ifdef LED_2_PORT
  HUS_LEDINIT(2);
#endif
#ifdef DLED_2_PORT
  HUS_DLEDINIT(2);
#endif

#ifdef LED_3_PORT
  HUS_LEDINIT(3);
#endif
#ifdef DLED_3_PORT
  HUS_DLEDINIT(3);
#endif

#ifdef LED_4_PORT
  HUS_LEDINIT(4);
#endif
#ifdef DLED_4_PORT
  HUS_DLEDINIT(4);
#endif

#ifdef LED_5_PORT
  HUS_LEDINIT(5);
#endif
#ifdef DLED_5_PORT
  HUS_DLEDINIT(5);
#endif

#ifdef LED_6_PORT
  HUS_LEDINIT(6);
#endif
#ifdef DLED_6_PORT
  HUS_DLEDINIT(6);
#endif

#ifdef LED_7_PORT
  HUS_LEDINIT(7);
#endif
#ifdef DLED_7_PORT
  HUS_DLEDINIT(7);
#endif
};


#ifdef LED_0_PORT
/** \brief set Led 0 on  */
static inline void HUS_led_0_on()  { HUS_LEDON(0); }
/** \brief set Led 0 off */
static inline void HUS_led_0_off() { HUS_LEDOFF(0); }
#endif

#ifdef DLED_0_PORT
/** \brief set Led 0 off */
static inline void HUS_led_0_off() { HUS_DLEDOFF(0); }
/** \brief set Duo led 0 color  */
static inline void HUS_led_0_color(const LED_color_t color) {
  HUS_led_0_off();
  DLED_0_PORT |= color << DLED_0_FIRST_BIT;
}
#endif

#ifdef LED_1_PORT
/** \brief set Led 1 on  */
static inline void HUS_led_1_on()  { HUS_LEDON(1); }
/** \brief set Led 1 off  */
static inline void HUS_led_1_off() { HUS_LEDOFF(1); }
#endif

#ifdef DLED_1_PORT
static inline void HUS_led_1_off() { HUS_DLEDOFF(1); }
static inline void HUS_led_1_color(const LED_color_t color) {
  HUS_led_1_off();
  DLED_1_PORT |= color << DLED_1_FIRST_BIT;
}
#endif

#ifdef LED_2_PORT
/** \brief set Led 2 on  */
static inline void HUS_led_2_on()  { HUS_LEDON(2); }
static inline void HUS_led_2_off() { HUS_LEDOFF(2); }
#endif

#ifdef DLED_2_PORT
static inline void HUS_led_2_off() { HUS_DLEDOFF(2); }
static inline void HUS_led_2_color(const LED_color_t color) {
  HUS_led_2_off();
  DLED_2_PORT |= color << DLED_2_FIRST_BIT;
}
#endif

#ifdef LED_3_PORT
/** \brief set Led 3 on  */
static inline void HUS_led_3_on()  { HUS_LEDON(3); }
static inline void HUS_led_3_off() { HUS_LEDOFF(3); }
#endif

#ifdef DLED_3_PORT
static inline void HUS_led_3_off() { HUS_DLEDOFF(3); }
static inline void HUS_led_3_color(const LED_color_t color) {
  HUS_led_3_off();
  DLED_3_PORT |= color << DLED_3_FIRST_BIT;
}
#endif

#ifdef LED_4_PORT
/** \brief set Led 4 on  */
static inline void HUS_led_4_on()  { HUS_LEDON(4); }
static inline void HUS_led_4_off() { HUS_LEDOFF(4); }
#endif

#ifdef DLED_4_PORT
static inline void HUS_led_4_off() { HUS_DLEDOFF(4); }
static inline void HUS_led_4_color(const LED_color_t color) {
  HUS_led_4_off();
  DLED_4_PORT |= color << DLED_4_FIRST_BIT;
}
#endif

#ifdef LED_5_PORT
/** \brief set Led 5 on  */
static inline void HUS_led_5_on()  { HUS_LEDON(5); }
static inline void HUS_led_5_off() { HUS_LEDOFF(5); }
#endif

#ifdef DLED_5_PORT
static inline void HUS_led_5_off() { HUS_DLEDOFF(5); }
static inline void HUS_led_5_color(const LED_color_t color) {
  HUS_led_5_off();
  DLED_5_PORT |= color << DLED_5_FIRST_BIT;
}
#endif

#ifdef LED_6_PORT
/** \brief set Led 6 on  */
static inline void HUS_led_6_on()  { HUS_LEDON(6); }
static inline void HUS_led_6_off() { HUS_LEDOFF(6); }
#endif

#ifdef DLED_6_PORT
static inline void HUS_led_6_off() { HUS_DLEDOFF(6); }
static inline void HUS_led_6_color(const LED_color_t color) {
  HUS_led_6_off();
  DLED_6_PORT |= color << DLED_6_FIRST_BIT;
}
#endif

#ifdef LED_7_PORT
/** \brief set Led 7 on  */
static inline void HUS_led_7_on()  { HUS_LEDON(7); }
static inline void HUS_led_7_off() { HUS_LEDOFF(7); }
#endif

#ifdef DLED_7_PORT
static inline void HUS_led_7_off() { HUS_DLEDOFF(7); }
static inline void HUS_led_7_color(const LED_color_t color) {
  HUS_led_7_off();
  DLED_7_PORT |= color << DLED_7_FIRST_BIT;
}
#endif
/*******************************************************************/

#endif /*  __LED_DRV_H__ */
