/*********************************************************************
 **   Martial Chateauvieux
 **   HORUS UAV System
 **   13 Feb 2007
 **********************************************************************
 **
 **    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
 **    Chateauvieux
 **
 **    This file is part of the HORUS UAV System distribution.
 **
 **    HORUS UAV System is free software; you can redistribute it
 **    and/or modify it under the terms of the GNU General Public
 **    License as published by the Free Software Foundation; either
 **    version 2 of the License, or (at your option) any later version.
 **
 **    HORUS UAV System is distributed in the hope that it will be
 **    useful, but WITHOUT ANY WARRANTY; without even the implied
 **    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 **    See the GNU General Public License for more details.
 **
 **    You should have received a copy of the GNU General Public
 **    License along with the HORUS UAV System; if not, write to the
 **    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 **    Boston, MA 02111-1307 USA
 **
 *********************************************************************/
/** \file macros_io.h This file contains macros helping for the
 **    manipulation of IO PORTS and PINS. The macros in this file are
 **    independant from any particular hardware configuration. They
 **    rely on a external configuration files.
 **
 ** \par PIN manipulation macros
 **
 ** The Pin manipulation macros are the raw macros used to manipulate
 ** IO pin of avr MCUs. The allow to configure, set and query the IO
 ** pins. They rely on the particular definition of symbols (placed in
 ** a hardware config file)
 **
 ** see the macros_led.h file for macros specific to LEDs and SWITCHes
 ** manipulation.
 **
 *********************************************************************/

#ifndef __MACROS_IO_H__
#define __MACROS_IO_H__

// Set output
/** \def HUS_IO_SET_HIGH(io_x)
 ** \brief Set the IO pin \b io_x high
 ** \param io_x
 **/
#define HUS_IO_SET_HIGH(io_x)	(io_x ## _PORT |= _BV(io_x ## _BIT))
#define HUS_PIN_SET_HIGH(io_x)	(HUS_IO_SET_HIGH(io_x))

/** \def HUS_IO_SET_LOW(io_x)
 ** \brief Set the IO pin \b io_x low
 ** \param io_x
 **/
#define HUS_IO_SET_LOW(io_x)	((io_x ## _PORT) &= ~_BV(io_x ## _BIT))
#define HUS_PIN_SET_LOW(io_x)	(HUS_IO_SET_LOW(io_x))

/** \def HUS_IO_TGL(io_x)
 ** \brief Toggle the IO pin \b io_x
 ** \param io_x
 **/
#define HUS_IO_TGL(io_x)		((io_x ## _PORT) ^= _BV(io_x ## _BIT))


// Read input
/** \def HUS_IO_IS_HIGH(io_x)
 ** \brief != 0 if IO pin \b io_x is high
 ** \param io_x
 **/
#define HUS_IO_IS_HIGH(io_x)	(~io_x ## _PIN & _BV(io_x ## _BIT))
#define HUS_PIN_IS_HIGH(io_x)	(HUS_IO_IS_HIGH(io_x))

/** \def HUS_IO_IS_LOW(io_x)
 ** \brief != 0 if IO pin \b io_x is low
 ** \param io_x
 **/
#define HUS_IO_IS_LOW(io_x)	( ! (HUS_PIN_IS_HIGH(io_x)))
#define HUS_PIN_IS_LOW(io_x)	(HUS_IO_IS_LOW(io_x))



// Init:

/** \def HUS_IO_SET_INP(io_x)
 ** \brief Set IO direction of pin io_x to input.
 ** \param io_x
 **/
#define HUS_IO_SET_INP(io_x)		(io_x ## _DDR  &= ~_BV(io_x ## _BIT))

/** \def HUS_IO_SET_INP_PLUP(io_x)
 ** \brief Set IO direction of pin io_x to input and activate internal pull-up.
 ** \param io_x
 **/
#define HUS_IO_SET_INP_PLUP(io_x)	(HUS_IO_SET_INP(io_x),HUS_IO_SET_HIGH(io_x))

/** \def HUS_IO_SET_INP_PLUP(io_x)
 ** \brief Set IO direction of pin io_x to input and de-activate internal pull-up.
 ** \param io_x
 **/
#define HUS_IO_SET_INP_NOPLUP(io_x)	(HUS_IO_SET_INP(io_x),HUS_IO_SET_LOW(io_x))

/** \def HUS_IO_SET_OUTP(io_x)
 ** \brief Set IO direction of pin io_x to output.
 ** \param io_x
 **/
#define HUS_IO_SET_OUTP(io_x)		(io_x ## _DDR  |= _BV(io_x ## _BIT))

//#define HUS_PIN_INI_OUTP(io_x)	((io_x ## _DDR  |= _BV(io_x ## _BIT)),HUS_PIN_SET_LOW(io_x))
// Obsolete, use HUS_IO_SET_OUTP instead


//#define HUS_PIN_INI_INP(io_x)	((io_x ## _DDR  &= ~_BV(io_x ## _BIT)),HUS_PIN_SET_HIGH(io_x))
// Obsolete, use HUS_IO_SET_INP_PLUP instead
/********************************************************************/

#endif 		/* __MACROS_IO_H__ */
