/*********************************************************************
 **   Martial Chateauvieux
 **   HORUS UAV System
 **   13 Feb 2007
 **********************************************************************
 **
 **    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
 **    Chateauvieux
 **
 **    This file is part of the HORUS UAV System distribution.
 **
 **    HORUS UAV System is free software; you can redistribute it
 **    and/or modify it under the terms of the GNU General Public
 **    License as published by the Free Software Foundation; either
 **    version 2 of the License, or (at your option) any later
 **    version.
 **
 **    HORUS UAV System is distributed in the hope that it will be
 **    useful, but WITHOUT ANY WARRANTY; without even the implied
 **    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 **    PURPOSE.  See the GNU General Public License for more details.
 **
 **    You should have received a copy of the GNU General Public
 **    License along with the HORUS UAV System; if not, write to the
 **    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 **    Boston, MA 02111-1307 USA
 **
 *********************************************************************/

/** \file macros_led.h This file contains macros for the manipulation
 ** of LEDs and SWITCHes. It relies on the macros_io.h set of IO
 ** macros.
 **
 ** \par LED Manipulation macros
 **
 ** The LED manipulation macros are constructed on top of the general
 ** PIN manipulation macros.
 **
 ** \par SWITCH Manipulation macros
 **
 ** The SWITCH manipulation macros are constructed on top of the
 ** general PIN manipulation macros.
 **
 *********************************************************************/

#ifndef __MACROS_LED_H__
#define __MACROS_LED_H__

#include "macros_io.h"

/*********************************************************************
** IO PORTLed manipulation macros
*********************************************************************/
typedef enum {
    LED_OFF    = 0x00,
    LED_GREEN  = 0x01,
    LED_RED    = 0x02,
    LED_ORANGE = 0x03
} LED_color_t;

#if defined(LED_INVERTED) 
#define HUS_LEDON(x)		HUS_PIN_SET_HIGH(LED_ ## x)
#define HUS_LEDOFF(x)		HUS_PIN_SET_LOW(LED_ ## x)
#else
#define HUS_LEDON(x)		HUS_PIN_SET_LOW(LED_ ## x)
#define HUS_LEDOFF(x)		HUS_PIN_SET_HIGH(LED_ ## x)
#endif
#define HUS_LEDINIT(x)		(HUS_IO_SET_OUTP(LED_ ## x), HUS_LEDOFF(x))


#define HUS_DLEDINIT(x)		(DLED_ ## x ## _DDR  |= (0x03 << DLED_ ## x ## _FIRST_BIT))
#define HUS_DLEDOFF(x)		(DLED_ ## x ## _PORT &= ~(0x03 << DLED_ ## x ## _FIRST_BIT))
#define HUS_DLEDCOLO(x, color)	((HUS_DLEDOFF(x)), (DLED_ ## x ## _PORT |= (color << DLED_ ## x ## _BIT)))



/*********************************************************************
** BUTTON Manipulation macros
*********************************************************************/
// Input pi & Internal pullup
#define HUS_BUT_INIT(but_x)		HUS_IO_SET_INP_PLUP(but_x)

// Read
#define HUS_BUT_IS_PRESSED(but_x)	HUS_PIN_IS_HIGH(but_x)
#define HUS_BUT_IS_RELEASED(but_x)	HUS_PIN_IS_LOW(but_x)
/********************************************************************/

#endif /*  __LED_DRV_H__ */
