/*********************************************************************
**   Martial Chateauvieux
**   HORUS UAV System
**   23 Feb 2007
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

/** \file
 **   
 ** This file implements the UART macros used to write UART
 ** drivers. These macros are hardware independant.
 **
 ********************************************************************/

#ifndef __MACRO_UART_H__
#define __MACRO_UART_H__

#include <avr/io.h>

/*==========================================================
** BAUD RATE macros
**========================================================*/
#define ubrr(baudrate) ((uint16_t)((SYSCLK / (16 * (baudrate))) - 1))
#define HUS_UART_BRATE_110	ubrr(110ul)
#define HUS_UART_BRATE_300	ubrr(300ul)
#define HUS_UART_BRATE_600	ubrr(600ul)
#define HUS_UART_BRATE_1200	ubrr(1200ul)
#define HUS_UART_BRATE_2400	ubrr(2400ul)
#define HUS_UART_BRATE_4800	ubrr(4800ul)
#define HUS_UART_BRATE_9600	ubrr(9600ul)
#define HUS_UART_BRATE_14400	ubrr(14400ul)
#define HUS_UART_BRATE_19200	ubrr(19200ul)
#define HUS_UART_BRATE_28800	ubrr(28800ul)
#define HUS_UART_BRATE_38400	ubrr(38400ul)
#define HUS_UART_BRATE_57600	ubrr(57600ul)
#define HUS_UART_BRATE_76800	ubrr(76800ul)
#define HUS_UART_BRATE_115200	ubrr(115200ul)
/*========================================================*/

/*==========================================================
** PARITY macros for UART*_init()
**========================================================*/
#define HUS_UART_PAR_NONE	0x00
#define HUS_UART_PAR_EVEN	_BV(UPM1)
#define HUS_UART_PAR_ODD	(_BV(UPM1)|_BV(UPM0))
/*========================================================*/

/*==========================================================
** STOP bits macros for UART*_init()
**========================================================*/
#define HUS_UART_STPBIT_1	0x00
#define HUS_UART_STPBIT_2	_BV(USBS)
/*========================================================*/

/*==========================================================
** Define register set uniformely across architectures
** (Taken from libgpsdrone.a(uart.c))
**========================================================*/
#if defined(__AVR_ATmega128__)
/*==========================================================
** Mega 128 definitions
**========================================================*/
#define UART0_UBRRL	UBRR0L
#define UART0_UBRRH	UBRR0H
#define UART0_UCSRA	UCSR0A
#define UART0_UCSRB	UCSR0B
#define UART0_UCSRC	UCSR0C
#define UART0_UDR	UDR0

#define UART1_UBRRL	UBRR1L
#define UART1_UBRRH	UBRR1H
#define UART1_UCSRA	UCSR1A
#define UART1_UCSRB	UCSR1B
#define UART1_UCSRC	UCSR1C
#define UART1_UDR	UDR1

#define SIG_UART0_DATA_vect	USART0_UDRE_vect
#define SIG_UART0_RECV_vect	USART0_RX_vect

#define SIG_UART1_DATA_vect	USART1_UDRE_vect
#define SIG_UART1_RECV_vect	USART1_RX_vect

#define UART0_TXCIE	TXCIE0
#define UART0_RXCIE	RXCIE0
#define UART0_TXEN	TXEN0
#define UART0_RXEN	RXEN0
#define UART0_UDRIE	UDRIE0

#define UART1_TXCIE	TXCIE1
#define UART1_RXCIE	RXCIE1
#define UART1_TXEN	TXEN1
#define UART1_RXEN	RXEN1
#define UART1_UDRIE	UDRIE1

#define MCU_HAS_UART0
#define MCU_HAS_UART1

#elif defined(__AVR_ATmega8__)
/*==========================================================
** Mega 8 definitions
**========================================================*/
#define UART0_UBRRL	UBRRL
#define UART0_UBRRH	UBRRH
#define UART0_UCSRA	UCSRA
#define UART0_UCSRB	UCSRB
#define UART0_UCSRC	UCSRC
#define UART0_UDR	UDR

#define SIG_UART0_DATA_vect	USART_UDRE_vect
#define SIG_UART0_RECV_vect	USART_RXC_vect

#define UART0_TXCIE	TXCIE
#define UART0_RXCIE	RXCIE
#define UART0_TXEN	TXEN
#define UART0_RXEN	RXEN
#define UART0_UDRIE	UDRIE

#define MCU_HAS_UART0
#define MCU_HAS_NO_UART1

#elif defined(__AVR_ATmega162__)
/*==========================================================
** Mega 162 definitions
**========================================================*/
#define UART0_UBRRL	UBRR0L
#define UART0_UBRRH	UBRR0H
#define UART0_UCSRA	UCSR0A
#define UART0_UCSRB	UCSR0B
#define UART0_UCSRC	UCSR0C
#define UART0_UDR	UDR0

#define UART1_UBRRL	UBRR1L
#define UART1_UBRRH	UBRR1H
#define UART1_UCSRA	UCSR1A
#define UART1_UCSRB	UCSR1B
#define UART1_UCSRC	UCSR1C
#define UART1_UDR	UDR1

#define SIG_UART0_DATA_vect	USART0_UDRE_vect
#define SIG_UART0_RECV_vect	USART0_RX_vect

#define SIG_UART1_DATA_vect	USART1_UDRE_vect
#define SIG_UART1_RECV_vect	USART1_RX_vect

#define UART0_TXCIE	TXCIE0
#define UART0_RXCIE	RXCIE1
#define UART0_TXEN	TXEN0
#define UART0_RXEN	RXEN0
#define UART0_UDRIE	UDRIE0

//inline static void UART0_enable_udr_intr()      { UART0_UCSRB |= _BV(UDRIE); }
//inline static void UART0_disable_udr_intr()     { UART0_UCSRB &= ~_BV(UDRIE); }

#define UART1_TXCIE	TXCIE1
#define UART1_RXCIE	RXCIE1
#define UART1_TXEN	TXEN0
#define UART1_RXEN	RXEN0
#define UART1_UDRIE	UDRIE1

#define MCU_HAS_UART0
#define MCU_HAS_UART1

#endif
/*========================================================*/

#define HUS_SET_BRATE(uart, baudrate)	    \
		((uart ## _UBRRH = (0x0f00 & baudrate)>>8), \
    		 (uart ## _UBRRL = (0x00ff & baudrate)))

#define HUS_SET_PARITY(uart, parity, stopbits)	    (uart ## _UCSRC |= (parity|stop_bits))


/*==========================================================
** Enable/Disable Uart Reception
**========================================================*/
#define HUS_ENABLE_RX(uart)	(uart ## _UCSRB |= _BV(RXEN))
#define HUS_DISABL_RX(uart)	(uart ## _UCSRB &= ~_BV(RXEN))

/*==========================================================
** Enable/Disable Uart Reception ISR
**========================================================*/
#define HUS_ENABLE_RX_ISR(uart)	(uart ## _UCSRB |= _BV(RXCIE))
#define HUS_DISABL_RX_ISR(uart)	(uart ## _UCSRB &= ~_BV(RXCIE))


/*==========================================================
** Enable/Disable Uart Transmission
**========================================================*/
#define HUS_ENABLE_TX(uart)	(uart ## _UCSRB |= _BV(TXEN))
#define HUS_DISABL_TX(uart)	(uart ## _UCSRB &= ~_BV(TXEN))

/*==========================================================
** Enable/Disable Uart Transmission ISR
**========================================================*/
#define HUS_ENABLE_TX_ISR(uart)	(uart ## _UCSRB |= _BV(uart ## _UDRIE))
#define HUS_DISABL_TX_ISR(uart)	(uart ## _UCSRB &= ~_BV(uart ## _UDRIE))


#define HUS_DECLARE_C_QUEUE(q_name) 	extern xQueueHandle name
#define HUS_DEFINE_C_QUEUE(q_name) 	xQueueHandle q_name
#define HUS_INIT_C_QUEUE(q_name, size) 	(q_name = xCreateQueue(size, sizeof(char))))

#endif /* __MACROS_UART_H__ */
