/*********************************************************************
** Martial Chateauvieux
** Wind speed meter test
** 23 Mai 2007
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2007 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include <math.h>

#include "config.h"
#include "project_config.h"

#include "mydelay.h"
#include "tick.h"		// 5ms ticker
#include "watchdog.h"		// 5ms ticker

#include "drv_airspeed.h"
#include "drv_lcd.h"
#include "drv_led.h"

#define abs(n) 		((n) >= 0 ? (n) : -(n))

volatile HUS_AIRS_t HUS_AIRS_status;

HUS_lcd_type_t lcd_type = {LCD_ROWS, LCD_COLS};
int main(int argc, char**argv, char **env) {

  uint16_t last_tick = 0;
  unsigned int i;

  MCUCR |= _BV(SE); 		// Enable sleep
    
  //LCD Init
  HUS_lcd_init(&lcd_type);
  HUS_lcd_cmd(HUS_LCD_ON_DSPL);
  FILE *lcd = fdevopen(HUS_lcd_putc, NULL);

  // LED init
  HUS_led_init();
  HUS_led_0_color(LED_OFF);

  // 5ms ticker init
  TCK_init();

  // Enable interupts
  sei();

  
  //---- Init Boot --------------
  HUS_lcd_cls(lcd);

  int rowidx = 0;

  HUS_lcd_goto(rowidx++, 0);
  HUS_lcd_puts_P(PSTR("boot wspeedmeter 1.0"), lcd);

  HUS_lcd_goto(rowidx++, 0);
  HUS_lcd_puts_P(PSTR("startup ADC"), lcd);
  for(i = 0 ; i < 20 ; i++) {
    delay_ms(50);
  }
  //-----------------------------

  //---- Config ADC - Single ended ADC0
  HUS_lcd_goto(rowidx++, 0);
  HUS_lcd_puts_P(PSTR("calibrate ADC"), lcd);

  HUS_AIRS_init(&HUS_AIRS_status);
  HUS_AIRS_calibrate_adc(&HUS_AIRS_status);
  //-----------------------------

  //---- Init LCD layout --------
  HUS_lcd_cls(lcd);

  rowidx = 0;

  HUS_lcd_goto(rowidx++, 0);
  HUS_lcd_puts_P(PSTR("raw adc:       /1024"), lcd);

  HUS_lcd_goto(rowidx++, 4);
  HUS_lcd_puts_P(PSTR("adc:        V"), lcd);

  HUS_lcd_goto(rowidx++, 6);
  HUS_lcd_puts_P(PSTR("S:        m/s"), lcd);
  //-----------------------------

  unsigned int s = 0, s_count = 0;
  unsigned int s4 = 0, s4_count = 0;
  last_tick = TCK_5ms;
  while(1) {
    
    /*--------------------------------------------
    ** Process 5ms ticker
    **------------------------------------------*/
    if (last_tick != TCK_5ms) {
      last_tick = TCK_5ms;
      
      // Every 1/4s
      if (++s4_count == 50) {
	
	s4_count = 0;
	s4++;

	// Restart conversion
	HUS_AIRS_restart_conversion(&HUS_AIRS_status);
      }

      // Every 1s
      if (++s_count == 200) {
	s_count = 0;
	s++;
	HUS_lcd_goto(3, 14);
	fprintf_P(lcd, PSTR("%5us"), s);

      }

      continue;
    }	
    
    if (HUS_AIRS_status.adc_status.ready) {

      // Convert to Volts x 10e6
      // (ADCValue * VRef * 10e6) / 1024
      int32_t voltsx10e6 = HUS_adc_se_to_voltx10e6(HUS_AIRS_status.mean_value);// + HUS_AIRS_status.offset);

      // Display results
      rowidx = 0;

      HUS_lcd_goto(rowidx++, 8);
      fprintf_P(lcd, PSTR("%+5d"), HUS_AIRS_status.mean_value);// + HUS_AIRS_status.offset);

      HUS_lcd_goto(rowidx++, 8);
      fprintf_P(lcd, PSTR("%c%01ld.%04ld"),
		voltsx10e6 >= 0 ? '+' : '-',
		abs(voltsx10e6) / 1000000,
		(abs(voltsx10e6) % 1000000) /100);
      
/*       HUS_lcd_goto(rowidx++, 11); */
/*       fprintf_P(lcd, PSTR("%4ld"), adc_to_airspeed_ms(mean_value)); */
		
      
      int32_t airspeed_dms = HUS_AIRS_adc_to_airspeed_dms(HUS_AIRS_status.mean_value);// + HUS_AIRS_status.offset);
      HUS_lcd_goto(rowidx++, 9);
      fprintf_P(lcd, PSTR("%c%4ld.%1ld"),
		airspeed_dms >= 0 ? '+' : '-',
		abs(airspeed_dms) / 10,
		abs(airspeed_dms) % 10);
      

      continue;
    }

      sleep_mode();
  } /* while(1) */
} /* main */

